/**
* ShowCaseStandalone
* Copyright (C) 2012 Kellerkindt <copyright at kellerkindt.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
package com.kellerkindt.scs.commands;


import java.util.UUID;

import org.bukkit.command.CommandSender;
import org.bukkit.inventory.ItemStack;

import com.kellerkindt.scs.Properties;
import com.kellerkindt.scs.ShowCaseStandalone;
import com.kellerkindt.scs.exceptions.InsufficientPermissionException;
import com.kellerkindt.scs.exceptions.MissingOrIncorrectArgumentException;
import com.kellerkindt.scs.internals.Todo;
import com.kellerkindt.scs.internals.Todo.Type;
import com.kellerkindt.scs.shops.DisplayShop;
import com.kellerkindt.scs.utilities.Term;
import com.kellerkindt.scs.utilities.Utilities;

/**
 * @author Sorklin <sorklin at gmail.com>
 */
public class DisplayCmd extends GenericCmd {
    
    public DisplayCmd(ShowCaseStandalone scs, CommandSender cs, String args[]){
        super(scs, cs, args);
        this.permission = Properties.permCreateDisplay;
    }

    @Override
    public boolean execute() throws MissingOrIncorrectArgumentException, InsufficientPermissionException {
        if(errorCheck()) {
            return true;
        }

        if(scs.getConfiguration().getWorldsBlacklisted().contains(player.getWorld().getName())) {
            throw new InsufficientPermissionException( Term.BLACKLIST_WORLD.get() );
        }
        
        //MaterialData mat = null;
        ItemStack is = null;
        
        //We have optional number of arguments.  Lets parse through them.
        try {
        
            switch (args.length){
                case 0:
                case 1:
                    is = Utilities.getItemStack(player, "this");
                    break;
                case 2:
                    is = Utilities.getItemStack(player, args[1]);
                    break;
            }
            
        } catch (Exception e) {
            throw new MissingOrIncorrectArgumentException ();
        }

        if (is == null || is.getTypeId() == 0) {
        	throw new MissingOrIncorrectArgumentException (Term.ITEM_MISSING.get());
        }

        // if the id already exists, the ShopHandler will set a new one
        DisplayShop	shop	= new DisplayShop(UUID.randomUUID(), player.getUniqueId(), null, is.clone());
        
        shop.setAmount	(0);
        shop.setPrice	(0);
        
        scs.msgPlayer(player, next);
        scs.addTodo(player, new Todo(player, Type.CREATE, shop, 0, null));
        
        return true;
    }
}