/**
* ShowCaseStandalone
* Copyright (C) 2012 Kellerkindt <copyright at kellerkindt.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
package com.kellerkindt.scs.commands;

import org.bukkit.command.CommandSender;

import com.kellerkindt.scs.PlayerSession;
import com.kellerkindt.scs.Properties;
import com.kellerkindt.scs.ShowCaseStandalone;
import com.kellerkindt.scs.exceptions.InsufficientPermissionException;
import com.kellerkindt.scs.exceptions.MissingOrIncorrectArgumentException;
import com.kellerkindt.scs.utilities.Messaging;
import com.kellerkindt.scs.utilities.Term;


/**
 * @author Sorklin <sorklin at gmail.com>
 */
public class MessageCmd extends GenericCmd {
    
    public MessageCmd(ShowCaseStandalone scs, CommandSender cs, String args[]){
        super(scs, cs, args);
        this.permission = Properties.permUse;
    }

    @Override
    public boolean execute() throws MissingOrIncorrectArgumentException, InsufficientPermissionException {
        if(errorCheck())
            return true;
        
        // get the players session
        PlayerSession session = scs.getPlayerSessionHandler().getSession(player);
        
        //This is the catch all message (meaning true or false isn't specified):
        String msg = session.showTransactionMessage() 
                    ? Term.IGNORE_RECEIVE.get("receiving")
                    : Term.IGNORE_RECEIVE.get("ignoring");
        
        //Overwrite if args[1] = true or false
        if(args.length > 1)
            if(args[1].equalsIgnoreCase("ignore")){
            	session.setShowTransactionMessage(false);
                msg = Term.IGNORE_TRANSACTION.get("ignored");
                
            } else if (args[1].equalsIgnoreCase("receive")) {
            	session.setShowTransactionMessage(true);
                msg = Term.IGNORE_TRANSACTION.get("received");
            } //msg: changeIgnoreReceive %1 for both
        
        Messaging.send(player, msg);
        return true;
    }
}
