/**
* ShowCaseStandalone
* Copyright (C) 2012 Kellerkindt <copyright at kellerkindt.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
package com.kellerkindt.scs.commands;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;

import org.bukkit.command.CommandSender;

import com.kellerkindt.scs.Properties;
import com.kellerkindt.scs.ShowCaseStandalone;
import com.kellerkindt.scs.exceptions.InsufficientPermissionException;
import com.kellerkindt.scs.exceptions.MissingOrIncorrectArgumentException;
import com.kellerkindt.scs.interfaces.ShopHandler;
import com.kellerkindt.scs.interfaces.StorageHandler;
import com.kellerkindt.scs.shops.Shop;
import com.kellerkindt.scs.utilities.Messaging;
import com.kellerkindt.scs.utilities.Term;

/**
 * @author Sorklin <sorklin at gmail.com>
 */
public class PruneCmd extends GenericCmd {
    
    public PruneCmd(ShowCaseStandalone scs, CommandSender cs, String args[]){
        super(scs, cs, args);
        this.mustBePlayer = false;
        this.permission = Properties.permAdmin;
    }

    @Override
    public boolean execute() throws MissingOrIncorrectArgumentException, InsufficientPermissionException {
        if(errorCheck())
            return true;
        
        //Purge command will empty the mini file, and then replace with whatever is loaded.
        //
        ShopHandler 				sh 			= scs.getShopHandler();
        StorageHandler<ShopHandler>	storage		= scs.getShopStorageHandler();
        
        try {
                Messaging.send(cs, Term.PRUNE.get());
                ShowCaseStandalone.slog(Level.INFO, "Backup.");
                List<Shop>	shops	= new ArrayList<Shop>();
                for (Shop p : sh)
                	shops.add(p);
                
                ShowCaseStandalone.slog(Level.INFO, "Remove all shops from storage.");
                sh.removeAll();
                
                ShowCaseStandalone.slog(Level.INFO, "Add backuped shops.");
                sh.addAll(shops);

                ShowCaseStandalone.slog(Level.INFO, "Saving all currently loaded shops.");
                storage.save(sh);
                
    	} catch (Exception ioe) {
    		ShowCaseStandalone.slog(Level.WARNING, "Exception on prune: " + ioe.getLocalizedMessage());
                Messaging.send(cs, Term.ERROR_GENERAL.get("pruning") + ioe.getLocalizedMessage());//msg: generalError %1 (reloading)
    	}
        
        return true;
    }   
}
